import { css } from "@linaria/core";
import { defineComponent, onMounted, onUnmounted } from "vue";
import { useEditor } from "../../../..";
import { TransferCtrl } from "@/modules/editor/controllers/TransferCtrl";
import { CompToolbars } from "@/modules/editor/objects/Toolbars";

const btnStyles = {
  top: {
    top: 0,
    left: "50%",
    transform: "translate(-50%, -50%)",
  },
  bottom: {
    bottom: 0,
    left: "50%",
    transform: "rotate(180deg) translate(50%, -50%)",
  },
  left: {
    top: "50%",
    left: 0,
    transform: "rotate(-90deg) translate(50%, -50%)",
  },
  right: {
    top: "50%",
    right: 0,
    transform: "rotate(90deg) translate(-50%, -50%)",
  },
};

export const Transfer = defineComponent({
  setup() {
    const editor = useEditor();
    const { store } = editor;
    const transferCtrl = new TransferCtrl(editor);
    const { transferStyle } = transferCtrl;

    onMounted(() => {
      setTimeout(() => {
        transferCtrl.init();
      });
    });

    onUnmounted(() => {
      transferCtrl.destroy();
    });

    onUnmounted(() => {
      transferCtrl.resetStyle();
    });

    return () => {
      const comp = store.currComp;
      return (
        transferStyle.width && (
          <div
            class={rootStyle}
            style={{
              top: transferStyle.top,
              left: transferStyle.left,
              width: transferStyle.width,
            }}
          >
            <div
              class={borderStyle}
              style={{
                width: transferStyle.width,
                height: transferStyle.height,
              }}
            ></div>
            <div
              class={resizeStyle}
              style={{ marginBottom: "-" + transferStyle.height }}
              onMousedown={(e) => transferCtrl.mousedown(e, "resize")}
            ></div>
            <div class={toolbarStyle}>
              {CompToolbars.default.map((item, i) => {
                return item.getVisible.call(editor, comp) ? (
                  <item.component
                    class="p-4px"
                    value={item.getValue?.(comp)}
                    onClick={() => item.onClick.call(editor, comp)}
                  />
                ) : null;
              })}
            </div>

            <div
              class={transformBtnsStyle}
              style={{ marginBottom: "-" + transferStyle.height }}
            >
              <div
                class={transBtnStyle}
                onMousedown={(e) => transferCtrl.mousedown(e, "move")}
              >
                +
              </div>
              <div
                class={transBtnStyle}
                onMousedown={(e) => transferCtrl.mousedown(e, "rotate")}
              >
                -
              </div>
            </div>
            {/* {Object.entries(btnStyles).map(([name, style]) => (
            <div
              class={[fanBtnStyle, "drag-disable"]}
              style={style}
              onMousedown={(e) =>
                transferCtrl.mousedown(
                  e,
                  `offset_${name as keyof typeof btnStyles}`
                )
              }
            >
              ↑
            </div>
          ))} */}
          </div>
        )
      );
    };
  },
});

const rootStyle = css`
  position: absolute;
`;

const borderStyle = css`
  position: absolute;
  top: 0;
  left: 0;
  width: 100%;
  height: 100%;
  outline: 2px solid @inf-primary-color;
  pointer-events: none;
  z-index: 999;
`;

const resizeStyle = css`
  position: absolute;
  bottom: 0;
  right: 0;
  width: 16px;
  height: 16px;
  border-radius: 50%;
  background-color: #fff;
  z-index: 999;
  transform: translate(50%, 50%);
  box-shadow: 0 0 2px 2px rgba(0, 0, 0, 0.2);
  cursor: nwse-resize;
  &:hover {
    border-color: @inf-primary-color;
  }
`;

const transformBtnsStyle = css`
  @apply space-x-5px whitespace-nowrap;
  position: absolute;
  bottom: 0;
  left: 50%;
  font-size: 16px;
  z-index: 999;
  transform: translate(-50%, 40px);
`;

const transBtnStyle = css`
  display: inline-block;
  width: 30px;
  height: 30px;
  border-radius: 50%;
  background-color: #fff;
  text-align: center;
  line-height: 30px;
  font-size: 16px;
  @apply shadow cursor-move;

  &:hover {
    background-color: @inf-primary-color;
  }
`;

const fanBtnStyle = css`
  position: absolute;
  width: 30px;
  height: 30px;
  border-radius: 50%;
  background-color: rgba(255, 255, 255, 0.3);
  font-size: 12px;
  text-align: center;
  clip-path: polygon(0 0, 100% 0, 100% 50%, 0 50%);
  cursor: grab;
  z-index: 999;

  &:hover {
    background: @inf-primary-color;
  }
`;

const toolbarStyle = css`
  @apply bg-white shadow rounded space-x-4px p-4px whitespace-nowrap;
  position: absolute;
  top: 0;
  left: 50%;
  transform: translate(-50%, -40px);
  z-index: 999;
`;
