import { DesignComp } from "@/modules/editor/objects/DesignTemp/DesignComp";
import { css } from "@linaria/core";
import { defineUI } from "queenjs";
import { onUnmounted, reactive } from "vue";
import { Container, Draggable } from "vue-dndrop";
import { useEditor } from "../../..";
import { HotKeyCtrl } from "../../../controllers/HotKeyCtrl";
import { CompUI } from "../../CompUI";
import { Transfer } from "../../CompUI/basicUI/Transfer";

export default defineUI({
  setup() {
    const editor = useEditor();
    const { store, actions, helper, components } = editor;

    const hotKeyCtrl = new HotKeyCtrl(editor);
    hotKeyCtrl.init();
    onUnmounted(() => {
      hotKeyCtrl.destroy();
    });

    const state = reactive({
      draging: false,
    });

    return () => {
      const pageRoot = helper.findRootComp();
      if (!pageRoot) return;
      return store.isEditMode ? (
        <CompUI.Page.Component class={contentStyle} compId={pageRoot.id}>
          {{
            Container(children: any) {
              return (
                <>
                  <Container
                    class="!min-h-750px"
                    group-name="canvas"
                    onDrop={(e: any) => {
                      if (e.payload) {
                        actions.addCompToDesign(e.payload, e.addedIndex);
                      } else {
                        actions.moveComp(e.removedIndex, e.addedIndex);
                      }
                    }}
                    onDragStart={() => (state.draging = true)}
                    onDragEnd={() => (state.draging = false)}
                    non-drag-area-selector={".drag-disable"}
                  >
                    {children}
                  </Container>
                  {store.currCompId &&
                    store.currCompId !== "root" &&
                    !state.draging && <Transfer key={store.currCompId} />}
                </>
              );
            },
            CompItem(comp: DesignComp) {
              const Comp = CompUI[comp.compKey]?.Component;
              return (
                <Draggable class="!flex flex-col">
                  <Comp compId={comp.id} />
                </Draggable>
              );
            },
          }}
        </CompUI.Page.Component>
      ) : (
        <components.Preview />
      );
    };
  },
});

const contentStyle = css`
  position: relative;
  user-select: none;

  .dndrop-container.vertical > .dndrop-draggable-wrapper {
    overflow: unset;
  }
`;
