import { Exception, queenApi } from "queenjs";
import { EditorModule } from "..";
import { ICompKeys } from "../../typings";
import { DesignComp } from "../../objects/DesignTemp/DesignComp";
import { CompUI } from "../../components/CompUI";

export const editActions = EditorModule.action({
  // 添加组件到画布
  addCompToDesign(compKey: ICompKeys, index?: number) {
    let compId: string;
    if (index === undefined && this.store.currComp?.compKey === "Container") {
      compId = this.store.insertCompContainer(compKey, this.store.currComp);
    } else {
      compId = this.store.insertDesignContent(compKey, index);
    }
    this.actions.pickComp(compId);
  },
  // 切换当前组件
  pickComp(compId: string) {
    this.store.setCurrComp(compId);
  },
  // 切换到父组件
  pickParentComp(compId: string) {
    const parentComp = this.helper.findParentComp(compId);
    parentComp && this.store.setCurrComp(parentComp.id);
  },
  // 删除组件
  removeComp(compId: string) {
    if (compId === this.store.currCompId) {
      this.store.setCurrComp("");
    }
    this.store.deleteComp(compId);
  },
  // 移动组件
  moveComp(selIndex: number, targetIndex: number) {
    if (selIndex === targetIndex) return;
    this.store.moveComp(selIndex, targetIndex);
  },
  // 保存项目
  async saveDesign() {
    try {
      queenApi.showLoading("保存中");
      this.store.clearUnusedComps();
      await this.https[this.store.isEditPage ? "saveDesign" : "saveComp"](
        this.store.designData
      );
      queenApi.messageSuccess("保存成功");
    } catch (error: any) {
      throw Exception.error(error.toString());
    } finally {
      queenApi.hideLoading();
    }
  },

  // 设置组件显示隐藏
  setCompPosition(comp: DesignComp) {
    comp.layout.position =
      comp.layout.position === "absolute" ? undefined : "absolute";
  },
  // 设置组件显示隐藏
  setCompVisible(comp: DesignComp) {
    comp.layout.visible = comp.layout.visible === false ? true : false;
  },
  // 清除组件变换
  clearCompTransform(comp: DesignComp) {
    comp.layout.margin = "";
    comp.layout.transform = {};
  },
  // 设置组件锁定状态
  setCompLock(comp: DesignComp) {
    //
  },
  // 设置组件层级
  setCompLayer(comp: DesignComp, offset: number) {
    comp.layout.zIndex = Math.min(
      Math.max((comp.layout.zIndex || 0) + offset, 0),
      99
    );
  },
  // 宽度铺满
  fullCompWidth(comp: DesignComp) {
    comp.layout.size || (comp.layout.size = []);
    comp.layout.size[0] = 750;
  },
  //
  setCompAlign(comp: DesignComp, align: string) {
    comp.layout.alignSelf = align;
  },
});
